package org.implab.gradle.containers;

import org.gradle.api.DefaultTask;
import org.gradle.api.Plugin;
import org.gradle.api.Project;
import org.gradle.api.artifacts.Dependency;
import org.gradle.api.file.ProjectLayout;
import org.gradle.api.tasks.Copy;
import org.gradle.api.tasks.Delete;
import org.implab.gradle.containers.cli.ImageName;
import org.implab.gradle.containers.tasks.BuildImage;
import org.implab.gradle.containers.tasks.PushImage;
import org.implab.gradle.containers.tasks.SaveImage;

public abstract class ContainerPlugin implements Plugin<Project>, ProjectMixin {

    public static final String BUILD_GROUP = "build";

    public static final String ARCHIVE_CONFIGURATION = "archive";

    public void apply(Project project) {
        ProjectLayout layout = project.getLayout();

        var basePlugin = plugin(ContainerBasePlugin.class);
        var containerExtension = basePlugin.getContainerExtension();

        configuration(Dependency.DEFAULT_CONFIGURATION, Configurations.CONSUMABLE);
        configuration(ARCHIVE_CONFIGURATION, Configurations.RESOLVABLE);

        var processResourcesTask = task("processResources", Copy.class, t -> {
            t.setGroup(BUILD_GROUP);
            t.from(layout.getProjectDirectory().dir("src/main"));
            t.into(containerExtension.getContextDirectory());
        });

        var buildImageTask = task("buildImage", BuildImage.class, t -> {
            t.setGroup(BUILD_GROUP);
            t.dependsOn(processResourcesTask);

            t.getContextDirectory().set(containerExtension.getContextDirectory());
            t.getImageIdFile().set(project.getLayout().getBuildDirectory().file("iid.json"));

            t.getImageName().set(containerExtension.getImageName().map(ImageName::toString));
        });

        task("clean", Delete.class, t -> {
            t.delete(layout.getBuildDirectory());
        });

        task("build", DefaultTask.class, t -> {
            t.setGroup(BUILD_GROUP);
            t.dependsOn(buildImageTask);
        });

        task("pushImage", PushImage.class, t -> {
            t.dependsOn(buildImageTask);
            t.getImageName().set(buildImageTask.flatMap(BuildImage::getImageName));
        });

        task("saveImage", SaveImage.class, t -> {
            t.dependsOn(buildImageTask);
            t.getExportImages().add(buildImageTask.flatMap(BuildImage::getImageName));
        });

        project.getArtifacts().add(Dependency.DEFAULT_CONFIGURATION, buildImageTask);
    }
}
