package org.implab.gradle.containers.cli;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;

public class ProcessSpec {
    private final ProcessBuilder builder;

    private RedirectFrom inputRedirect;

    private RedirectTo outputRedirect;

    private RedirectTo errorRedirect;

    private List<String> command = new ArrayList<>();

    private File directory;

    public ProcessSpec() {
        builder = new ProcessBuilder();
    }

    public CompletableFuture<Integer> start() throws IOException {
        var tasks = new ArrayList<CompletableFuture<?>>();

        builder.command(this.command);
        builder.directory(directory);

        var proc = builder.start();

        tasks.add(proc.onExit());

        if (inputRedirect != null)
            tasks.add(inputRedirect.redirect(proc.getOutputStream()));

        if (outputRedirect != null)
            tasks.add(outputRedirect.redirect(proc.getInputStream()));

        if (errorRedirect != null)
            tasks.add(errorRedirect.redirect(proc.getErrorStream()));

        return CompletableFuture
                .allOf(tasks.toArray(new CompletableFuture<?>[0]))
                .thenApply(t -> proc.exitValue());
    }

    public void exec() throws InterruptedException, ExecutionException, IOException {
        var code = start().get();
        if (code != 0)
            throw new IOException("The process exited with error code " + code);
    }

    public List<String> command() {
        return this.command;
    }

    public ProcessSpec command(String... args) {
        this.command = new ArrayList<>();
        args(args);
        return this;
    }

    public ProcessSpec directory(File workingDir) {
        this.directory = workingDir;
        return this;
    }

    public ProcessSpec command(Collection<String> args) {
        this.command = new ArrayList<>();
        args(args);
        return this;
    }

    public ProcessSpec args(String... args) {
        for (String arg : args)
            this.command.add(arg);

        return this;
    }

    public ProcessSpec args(Collection<String> args) {
        this.command.addAll(args);

        return this;
    }

    public ProcessSpec redirectStderr(RedirectTo to) {
        this.errorRedirect = to;
        return this;
    }

    public ProcessSpec redirectStdin(RedirectFrom from) {
        this.inputRedirect = from;
        return this;
    }

    public ProcessSpec redirectStdout(RedirectTo to) {
        this.outputRedirect = to;
        return this;
    }
}
