package org.implab.gradle.containers.tasks;

import java.io.File;
import java.io.IOException;
import java.util.Optional;
import java.util.concurrent.ExecutionException;

import org.gradle.api.DefaultTask;
import org.gradle.api.GradleException;
import org.gradle.api.file.RegularFile;
import org.gradle.api.provider.Property;
import org.gradle.api.tasks.Input;
import org.gradle.api.tasks.Internal;
import org.implab.gradle.containers.ContainerExtension;
import org.implab.gradle.containers.PropertiesMixin;
import org.implab.gradle.containers.cli.ComposeTraits;
import org.implab.gradle.containers.cli.DockerTraits;
import org.implab.gradle.containers.cli.ProcessSpec;
import org.implab.gradle.containers.cli.RedirectFrom;
import org.implab.gradle.containers.cli.RedirectTo;
import org.implab.gradle.containers.cli.Utils;

public abstract class DockerCliTask extends DefaultTask implements PropertiesMixin {

    @Input
    public abstract Property<String> getCliCmd();

    /**
     * Returns working directory for docker commands
     */
    @Input
    @org.gradle.api.tasks.Optional
    public abstract Property<File> getWorkingDirectory();

    @Internal
    protected ContainerExtension getContainerExtension() {
        return getProject().getExtensions().getByType(ContainerExtension.class);
    }

    public DockerCliTask() {
        getCliCmd().convention(getContainerExtension().getCliCmd());

    }

    protected Optional<RedirectTo> loggerInfoRedirect() {
        return getLogger().isInfoEnabled() ? Optional.of(RedirectTo.consumer(getLogger()::info)) : Optional.empty();
    }

    protected Optional<RedirectTo> loggerErrorRedirect() {
        return getLogger().isErrorEnabled() ? Optional.of(RedirectTo.consumer(getLogger()::error)) : Optional.empty();
    }

    protected Optional<RedirectTo> stdoutRedirection() {
        return loggerInfoRedirect();
    }

    protected Optional<RedirectTo> stderrRedirection() {
        return loggerErrorRedirect();
    }

    protected Optional<RedirectFrom> stdinRedirection() {
        return Optional.empty();
    }

    protected TaskDockerTraits docker() {
        return new TaskDockerTraits();
    }

    protected boolean imageExists(String imageId) {
        try {
            return checkRetCode(docker().imageExists(imageId), 0);
        } catch (InterruptedException | ExecutionException | IOException e) {
            // wrap to unchecked exception
            throw new GradleException("Failed to execute imageExists", e);
        }
    }

    protected boolean containerExists(String containerId) {
        try {
            return checkRetCode(docker().containerExists(containerId), 0);
        } catch (InterruptedException | ExecutionException | IOException e) {
            // wrap to unchecked exception
            throw new GradleException("Failed to execute imageExists", e);
        }
    }

    protected void exec(ProcessSpec spec) throws InterruptedException, ExecutionException, IOException {

        stdoutRedirection().ifPresent(spec::redirectStdout);
        stderrRedirection().ifPresent(spec::redirectStderr);
        stdinRedirection().ifPresent(spec::redirectStdin);

        getLogger().info("Staring: {}", spec.command());

        // runs the command and checks the error code
        spec.exec();
    }

    protected boolean checkRetCode(ProcessSpec proc, int code)
            throws InterruptedException, ExecutionException, IOException {
        if (getLogger().isInfoEnabled()) {
            proc.redirectStdout(RedirectTo.consumer(getLogger()::info))
                    .redirectStderr(RedirectTo.consumer(getLogger()::info));
        }

        getLogger().info("Starting: {}", proc.command());

        return proc.start().get() == code;
    }

    /**
     * Helper function to read 
     * @param idFile
     * @return
     */
    protected String readId(File idFile) {
        if (idFile.exists()) {
            try {
                return Utils.readImageRef(idFile).getId();
            } catch (IOException e) {
                getLogger().error("Failed to read imageId {}: {}", idFile, e);
                return null;
            }
        } else {
            return null;
        }
    }

    protected ProcessSpec commandBuilder(String... command) {
        var spec = new ProcessSpec().args(getCliCmd().get()).args(command);

        if (getWorkingDirectory().isPresent())
            spec.directory(getWorkingDirectory().get());

        return spec;
    }

    class TaskDockerTraits extends DockerTraits {

        @Override
        protected ProcessSpec builder(String... command) {
            return commandBuilder(command);
        }

        public ComposeTraits compose(ComposeTaskMixin props) {
            return compose(props.getComposeFile().map(RegularFile::getAsFile).get(), props.getProfiles().get());
        }

    }
}
