package org.implab.gradle.containers.tasks;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ExecutionException;

import org.gradle.api.Action;
import org.gradle.api.GradleException;
import org.gradle.api.file.RegularFileProperty;
import org.gradle.api.provider.Property;
import org.gradle.api.tasks.Internal;
import org.gradle.api.tasks.TaskAction;
import org.implab.gradle.containers.dsl.VolumeSpec;

public abstract class RunContainer extends BaseExecTask implements ContainerTaskMixin {

    private boolean transientContainer = true;

    private boolean startIfExists = true;

    @Internal
    public abstract Property<Object> getImageName();

    @Internal
    public abstract Property<String> getContainerName();

    /**
     * Transient containers will be removed after completion. Default is `true`
     */
    @Internal
    public boolean isTransientContainer() {
        return transientContainer;
    }

    public void setTransientContainer(boolean value) {
        transientContainer = value;
    }

    @Internal
    public boolean getStartIfExists() {
        return startIfExists;
    }

    public void setStartIdExists(boolean value) {
        startIfExists = value;
    }

    /**
     * Specified the file where the stared container id will be written.
     * Default value is a temporary file.
     * 
     * <p>
     * This property can be use in tasks where the container id is required
     * to perform operation, for example to StopContainer tasks.
     */
    @Internal
    public abstract RegularFileProperty getContainerIdFile();

    public RunContainer() {
        getContainerIdFile().convention(() -> new File(getTemporaryDir(), "cid"));
    }

    /**
     * Adds volume specification
     * 
     * @param configure
     */
    public void volume(Action<VolumeSpec> configure) {
        getOptions().add("-v");
        getOptions().add(provider(() -> {
            var volumeSpec = getObjectFactory().newInstance(VolumeSpec.class);
            configure.execute(volumeSpec);
            return volumeSpec.resolveSpec();
        }));
    }

    @TaskAction
    public void run() throws InterruptedException, IOException, ExecutionException {
        var params = new ArrayList<>(getOptions().get());

        if (getContainerId().map(this::containerExists).getOrElse(false)) {
            var cid = getContainerId().get();
            if (getStartIfExists()) {
                getLogger().info("Container {} already created, starting.", cid);

                if (isInteractive())
                    params.add("--interactive");
                if (!isDetached())
                    params.add("--attach");

                exec(docker().containerStart(Set.of(cid), params));
            } else {
                throw new GradleException("Container " + cid + " already exists");
            }
        } else {

            if (isInteractive())
                params.add("--interactive");
            if (isTransientContainer())
                params.add("--rm");
            if (isDetached())
                params.add("--detach");
            if (getAllocateTty())
                params.add("--tty");
            if (getContainerName().isPresent())
                params.addAll(List.of("--name", getContainerName().get()));

            if (getContainerIdFile().isPresent()) {
                // delete previous container id otherwise the container will fail to start
                getContainerIdFile().getAsFile().get().delete();

                params.addAll(List.of("--cidfile", getContainerIdFile().get().getAsFile().toString()));
            }

            exec(docker().containerRun(
                    getImageName().map(Object::toString).get(),
                    params,
                    getCommandLine().get()));
        }
    }

}
