package org.implab.gradle.containers.tasks;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Optional;
import java.util.concurrent.ExecutionException;

import org.gradle.api.file.DirectoryProperty;
import org.gradle.api.file.FileCollection;
import org.gradle.api.file.RegularFile;
import org.gradle.api.provider.Property;
import org.gradle.api.provider.Provider;
import org.gradle.api.provider.SetProperty;
import org.gradle.api.tasks.Input;
import org.gradle.api.tasks.Internal;
import org.gradle.api.tasks.OutputFile;
import org.gradle.api.tasks.TaskAction;
import org.gradle.api.tasks.TaskExecutionException;
import org.implab.gradle.containers.cli.Utils;

public abstract class SaveImage extends DockerCliTask {

    @Input
    public abstract SetProperty<Object> getExportImages();

    @OutputFile
    public Provider<RegularFile> getArchiveFile() {
        return getDestinationDirectory().file(getArchiveFileName());
    }

    @Internal
    public abstract DirectoryProperty getDestinationDirectory();

    @Internal
    public abstract Property<String> getArchiveFileName();

    @Internal
    public abstract Property<String> getArchiveBaseName();

    @Internal
    public abstract Property<String> getArchiveVersion();

    @Internal
    public abstract Property<String> getArchiveClassifier();

    @Internal
    public abstract Property<String> getArchiveExtension();

    String readImageRefTag(File file) throws Exception {
        getLogger().info("Reading image ref from {}", file);
        var imageRef = Utils.readImageRef(file);
        return imageRef.getTag().orElseThrow(() -> new Exception("The image tag is required to save image"));
    }

    public void imageRef(File file) {
        getExportImages().add(provider(() -> {
            return readImageRefTag(file);
        }));
    }

    public void imageRefs(FileCollection files) {
        dependsOn(files);
        getExportImages().addAll(provider(() -> {
            var tags = new ArrayList<String>();
            for (File file : files) {
                tags.add(readImageRefTag(file));
            }
            return tags;
        }));
    }

    public SaveImage() {
        getArchiveFileName().convention(provider(this::conventionalArchiveFileName));
        getArchiveBaseName().convention(provider(this::conventionalArchiveBaseName));
        getArchiveVersion().convention(provider(() -> getProject().getVersion()).map(x -> x.toString()));
        getDestinationDirectory().convention(getProject().getLayout().getBuildDirectory());
        getArchiveExtension().convention("tar");
    }

    private String conventionalArchiveBaseName() {
        ArrayList<String> parts = new ArrayList<>();
        Optional.of(getProject().getGroup()).map(x -> x.toString()).ifPresent(parts::add);
        parts.add(getProject().getName());
        return String.join("-", parts);
    }

    private String conventionalArchiveFileName() {
        ArrayList<String> parts = new ArrayList<>();

        if (getArchiveBaseName().isPresent())
            parts.add(getArchiveBaseName().get());

        if (getArchiveVersion().isPresent())
            parts.add(getArchiveVersion().get());

        if (getArchiveClassifier().isPresent())
            parts.add(getArchiveClassifier().get());

        if (parts.size() == 0)
            throw new TaskExecutionException(this, new Exception("The archiveFileName ism't specified"));

        return String.join("-", parts) + "." + getArchiveExtension().get();
    }

    @TaskAction
    public void run() throws InterruptedException, IOException, ExecutionException {
        exec(docker().imageSave(
                getExportImages().map(Utils::mapToString).get(),
                getArchiveFile().map(RegularFile::getAsFile).get()));
    }
}
